# -*- coding: utf-8 -*-
"""libsqs module

Module interface for dealing with SQS queue and plus some useful dispatching batch jobs impl

https://boto3.amazonaws.com/v1/documentation/api/latest/guide/sqs-example-sending-receiving-msgs.html
https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/sqs.html
"""
import logging
import uuid
from typing import List

from libumccr import libjson
from libumccr.aws import sqs_client

logger = logging.getLogger(__name__)

MAX_BATCH_SIZE = 10


def _build(queue_arn):
    client = sqs_client()
    queue_url = client.get_queue_url(QueueName=arn_to_name(queue_arn))['QueueUrl']
    return client, queue_url


def arn_to_name(arn: str):
    """Get queue name from given SQS ARN"""
    arr = arn.split(':')
    if isinstance(arr, list):
        return arr[-1]
    return None


def dispatch_notification(queue_arn: str, message: dict, group_id: str):
    """
    Note: backing notification queue is FIFO Delay queue with ContentBasedDeduplication enabled.
    Hence, if sha256(message) is the same, it will get dedup.
    If group_id is set to the same value, message will be enqueue in FIFO order.

    The main use case here is to serialize simultaneous distributed event messages into a batch notification aggregate.

    :param queue_arn:
    :param message:
    :param group_id:
    :return:
    """

    response = enqueue_message(
        queue_arn=queue_arn,
        MessageBody=libjson.dumps(message),
        MessageGroupId=group_id,
    )

    logger.info(f"NOTIFICATION QUEUE RESPONSE: \n{libjson.dumps(response)}")
    return response


def dispatch_jobs(queue_arn, job_list, batch_size=10, fifo=True):
    """
    Queue job in batch of given size

    See [1] for message format which to be consumed by SQS Lambda trigger

    If it is on SQS Lambda trigger consumer, less batch_size mean more parallel lambda invocations i.e. it goes
    as low as 1 message per lambda invocation. Messages enqueue within the same group_id are guaranteed FIFO order.
    Also guaranteed delivery-once and deduplicate based on hash(message body content).

    Example use case for how to adjust batch_size to Lambda concurrency:
    Say, calling WES endpoint for workflow launch take approx. 1s
    Then, 10 messages per batch for launching 10 workflows = 1s * 10 (+ headroom for warmup, complexity) = say est. 20s

    Default Lambda execution timeout is 6s -- configurable upto 900s.
    However typical Lambda prefer timeout setting is 30s, or less.
    Also note that SQS the default visibility timeout for a message is 30 seconds [2].
    [1] say to set the source queue's visibility timeout to at least 6 times the timeout that of Lambda function.

    Hence observe and do Maths around this for optimal operational setting.

    [1]: https://docs.aws.amazon.com/lambda/latest/dg/with-sqs.html
    [2]: https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-visibility-timeout.html

    :param queue_arn:
    :param job_list:
    :param batch_size:
    :param fifo: default True
    :return:
    """
    _batch_size = batch_size if batch_size < MAX_BATCH_SIZE else MAX_BATCH_SIZE
    _chunks = [job_list[x:x + _batch_size] for x in range(0, len(job_list), _batch_size)]
    responses = {}
    for chunk in _chunks:
        entries = []
        group_id = str(uuid.uuid4())
        for job in chunk:
            entry = {
                'Id': str(uuid.uuid4()),
                'MessageBody': libjson.dumps(job),
            }
            if fifo:
                entry['MessageGroupId'] = group_id
            entries.append(entry)
        resp = enqueue_messages(queue_arn, entries)
        responses[group_id] = {k: v for k, v in resp.items() if k.startswith('Successful') or k.startswith('Failed')}

    logger.info(f"JOB QUEUE RESPONSE: \n{libjson.dumps(responses)}")
    return responses


def enqueue_messages(queue_arn: str, entries: List[dict]):
    """
    Enqueue batch message entries into given queue

    :param queue_arn:
    :param entries:
    :return:
    """
    client, queue_url = _build(queue_arn=queue_arn)
    return client.send_message_batch(QueueUrl=queue_url, Entries=entries)


def enqueue_message(queue_arn: str, **kwargs):
    """
    Enqueue a message

    :param queue_arn:
    :param kwargs:
    :return:
    """
    client, queue_url = _build(queue_arn=queue_arn)
    return client.send_message(QueueUrl=queue_url, **kwargs)



# -*- coding: utf-8 -*-
"""aws package

Factory init module for creating boto3 client and resource to interface AWS services
"""
import boto3


def session(**kwargs):
    """
    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/core/session.html

    :param kwargs:
    :return:
    """
    return boto3.session.Session(**kwargs)


def client(service_name, **kwargs):
    return session().client(service_name=service_name, **kwargs)


def resource(service_name, **kwargs):
    return session().resource(service_name=service_name, **kwargs)


def s3_client(**kwargs):
    return client('s3', **kwargs)


def sqs_client(**kwargs):
    return client('sqs', **kwargs)


def ssm_client(**kwargs):
    return client('ssm', **kwargs)


def sm_client(**kwargs):
    return client('secretsmanager', **kwargs)



# -*- coding: utf-8 -*-
"""libsm module

Module interface with AWS Secrets Manager and, LRU cache for hit less to AWS endpoint if any
"""
import base64
import logging

from cachetools.func import lru_cache

from libumccr.aws import sm_client

logger = logging.getLogger(__name__)


@lru_cache(maxsize=64)
def get_secret(secret_name):
    resp = sm_client().get_secret_value(
        SecretId=secret_name
    )

    if 'SecretString' in resp:
        return resp['SecretString']
    else:
        return base64.b64decode(resp['SecretBinary'])



from unittest import TestCase

from libumccr.aws import libsqs


class LibSQSUnitTests(TestCase):

    # TODO https://github.com/umccr/libumccr/issues/3

    def test_arn_to_name(self):
        """
        python -m unittest tests.aws.test_libsqs.LibSQSUnitTests.test_arn_to_name
        """
        q_name = libsqs.arn_to_name("arn:aws:sqs:ap-southeast-2:1234567890:data-portal-mock-queue.fifo")
        self.assertEqual(q_name, "data-portal-mock-queue.fifo")



# -*- coding: utf-8 -*-

# Version follows PEP 440 for PyPI publishing
# https://www.python.org/dev/peps/pep-0440/
__version__ = VERSION = '0.1.0rc1'

import logging

logging.getLogger(__name__).addHandler(logging.NullHandler())



# -*- coding: utf-8 -*-
"""libgdrive module

Module interface for underlay Google Drive and Sheet common operations
"""
import json
import logging
from typing import List

import gspread
import pandas as pd
from google.oauth2 import service_account
from gspread_pandas import Spread

logger = logging.getLogger(__name__)


DEFAULT_SHEET_NAME = "Sheet1"
FAILED_SHEET_NAME = "Failed Runs"
_read_scopes = ['https://www.googleapis.com/auth/drive.readonly']
_write_scopes = ['https://www.googleapis.com/auth/drive']  # TODO: limit to write?


def download_sheet1_csv(account_info: str, file_id: str) -> bytes:
    """
    Get a Google drive Spreadsheet file Sheet1 as exported in CSV format
    Required a Service Account with Read-Only permission on the requested file resource

    Limitation: Only first Sheet

    :param account_info:
    :param file_id:
    :return file content: in csv format bytes
    """

    gc = gspread.service_account_from_dict(json.loads(account_info), scopes=_read_scopes)
    sh: gspread.Spreadsheet = gc.open_by_key(file_id)

    return pd.DataFrame(sh.sheet1.get_all_records()).to_csv().encode()


def download_sheet(account_info: str, file_id: str, sheet=None) -> pd.DataFrame:
    """Download the specified sheet from GDrive and return as panda DataFrame object

    :param account_info:
    :param file_id:
    :param sheet: str,int the sheet in the metadata spreadsheet to load
    :return dataframe: file content in panda dataframe, NOTE: it still return blank DF if exception occur
    """

    credentials = service_account.Credentials.from_service_account_info(json.loads(account_info))
    spread = Spread(spread=file_id, creds=credentials.with_scopes(_read_scopes))

    try:
        return spread.sheet_to_df(sheet=sheet, index=0, header_rows=1, start_row=1)
    except (gspread.exceptions.WorksheetNotFound, gspread.exceptions.APIError) as e:
        logger.warning(f"Returning empty data frame for sheet {sheet}. Exception: {type(e).__name__} -- {e}")
        return pd.DataFrame()


def append_records(account_info: str, file_id: str, data: List[tuple], sheet=DEFAULT_SHEET_NAME):
    gc = gspread.service_account_from_dict(json.loads(account_info), scopes=_write_scopes)
    sh: gspread.Spreadsheet = gc.open_by_key(file_id)

    params = {
        'valueInputOption': 'USER_ENTERED',
        'insertDataOption': 'INSERT_ROWS'
    }
    body = {
        'majorDimension': 'ROWS',
        'values': data
    }

    resp = sh.values_append(range=sheet, params=params, body=body)
    return resp



from unittest import TestCase, skip

from libumccr.aws import libssm


class LibSsmUnitTests(TestCase):
    # TODO https://github.com/umccr/libumccr/issues/4
    pass


class LibSsmIntegrationTests(TestCase):

    @skip
    def test_get_secret(self):
        """
        python -m unittest tests.aws.test_libssm.LibSsmIntegrationTests.test_get_secret
        """

        key = "/iap/jwt-token"

        value = libssm.get_secret(key=key)

        self.assertIsNotNone(value)
        self.assertIsInstance(value, str)
        # print(value)



from unittest import TestCase, skip

from libumccr.aws import libsm


class LibSmUnitTests(TestCase):
    # TODO https://github.com/umccr/libumccr/issues/2
    pass


class LibSmIntegrationTests(TestCase):

    @skip
    def test_get_secret(self):
        """
        python -m unittest tests.aws.test_libsm.LibSmIntegrationTests.test_get_secret
        """

        secret_name = "IcaSecretsPortal"

        secret = libsm.get_secret(secret_name=secret_name)

        self.assertIsNotNone(secret)
        self.assertIsInstance(secret, str)
        # print(secret)



# -*- coding: utf-8 -*-
"""libslack module

Module interface for underlay Slack client operations
Loosely based on design patterns: Facade, Adapter/Wrapper

Should retain/suppress all Slack API calls here, including
Slack API specific exceptions and data type that need for processing response.

Impl Note:

There are couple of ways to send Slack messages:
https://api.slack.com/messaging/sending#sending_methods

#1
For Slack Web API and Real Time Messaging (RTM) API, should utilise `python-slackclient`:
https://github.com/slackapi/python-slackclient

#2
For Slack web hook, apparently there is no official Python SDK[1] for Incoming Webhooks[2].
We could leverage package like `slack-webhook`[3]. However, we will be using built-in
`http.client` for now for back-porting existing code[4][5] purpose.

[1]: https://slack.dev
[2]: https://api.slack.com/messaging/webhooks
[3]: https://pypi.org/project/slack-webhook/
[4]: https://github.com/umccr/infrastructure/blob/master/cdk/apps/slack/lambdas/iap/notify_slack.py
[5]: https://github.com/umccr/infrastructure/blob/master/terraform/stacks/bootstrap/lambdas/notify_slack.py

If unsure, start with Pass-through call.
"""
import http.client
import logging
import os
from enum import Enum
from http import HTTPStatus
from typing import List

from libumccr import libjson
from libumccr.aws import libssm

headers = {
    'Content-Type': 'application/json',
}

logger = logging.getLogger(__name__)


def call_slack_webhook(sender, topic, attachments, **kwargs):

    slack_webhook_id = kwargs.get('webhook_id', None)
    if slack_webhook_id is None:
        slack_webhook_id = os.getenv('SLACK_WEBHOOK_ID', None)
    if slack_webhook_id is None:
        slack_webhook_id = libssm.get_ssm_param('/slack/webhook/id')

    assert slack_webhook_id is not None, "SLACK_WEBHOOK_ID is not defined"

    slack_webhook_endpoint = f"/services/{slack_webhook_id}"

    slack_host = kwargs.get('host', None)
    if slack_host is None:
        slack_host = os.getenv('SLACK_HOST', None)
    if slack_host is None:
        slack_host = "hooks.slack.com"

    slack_channel = kwargs.get('channel', None)
    if slack_channel is None:
        slack_channel = os.getenv('SLACK_CHANNEL', None)

    # NOTE: it strikes me that `elif` check for None won't work here! ;O
    # elif slack_channel is None:
    #     slack_channel = libssm.get_ssm_param('/data_portal/backend/slack_channel')

    if slack_channel is None:
        slack_channel = libssm.get_ssm_param('/data_portal/backend/slack_channel')

    assert slack_channel is not None, "SLACK_CHANNEL is not defined"

    connection = http.client.HTTPSConnection(slack_host)

    post_data = {
        "channel": slack_channel,
        "username": sender,
        "text": f"*{topic}*",
        "icon_emoji": ":aws_logo:",
        "attachments": attachments
    }
    logger.info(f"Slack POST data: {libjson.dumps(post_data)}")

    connection.request("POST", slack_webhook_endpoint, libjson.dumps(post_data), headers)
    response = connection.getresponse()
    logger.info(f"Slack webhook response status: {response.status}")
    connection.close()

    # see https://api.slack.com/messaging/webhooks#handling_errors
    if response.status == HTTPStatus.OK:
        return response.status
    else:
        return None


class SlackColor(Enum):
    GREEN = "#36A64F"
    RED = "#FF0000"
    BLUE = "#439FE0"
    GRAY = "#DDDDDD"
    BLACK = "#000000"
    ORANGE = "#FFA500"


class SlackField(object):
    pass


class SlackAttachment(object):
    pass


class SlackMessage(object):
    def __init__(self, sender, topic, attachments):
        self.sender: str = sender
        self.topic: str = topic
        self.attachments: List[SlackAttachment] = attachments



# -*- coding: utf-8 -*-
"""libssm module

Mainly interface with SSM Parameter Store and, LRU cache for hit less to AWS endpoint if any
"""
import logging

from cachetools.func import lru_cache

from libumccr.aws import ssm_client

logger = logging.getLogger(__name__)


@lru_cache(maxsize=64)
def get_secret(key) -> str:
    """
    Retrieve the secret value from SSM.
    :param key: the key of the secret
    :return: the secret value
    """
    resp = ssm_client().get_parameter(
        Name=key,
        WithDecryption=True
    )
    return resp['Parameter']['Value']


def get_ssm_param(name):
    """
    Fetch the parameter with the given name from SSM Parameter Store.
    """
    return get_secret(name)


class SSMParamStore(object):
    def __init__(self, key):
        self._key = key
        self._value = None

    @property
    def key(self):
        return self._key

    @property
    def value(self):
        assert self._key is not None, "Undefined key"
        return get_secret(self._key)

    def get_value(self):
        return self.value

    get: get_value

    def __str__(self):
        return f"{self._key}"



# -*- coding: utf-8 -*-
"""libs3 module

Module interface for underlay S3 client operations
Loosely based on design patterns: Facade, Adapter/Wrapper

Should retain/suppress all boto S3 API calls here, including
boto specific exceptions and, boto specific data type that need
for processing response.

Goal is, so that else where in code, it does not need to depends on boto3
API directly. i.e. No more import boto3, but just import libs3 instead.

If unsure, start with Pass-through call.

What impl should include in this module:
    Consider, we can take this libs3 to another application and, these impls
    are still meaningful in that totally different application context there!
    Consider, we can package and distribute these impls as standalone SDK/lib.
"""
import gzip
import logging
from datetime import datetime
from enum import Enum

from botocore.exceptions import ClientError

from libumccr.aws import s3_client, resource

logger = logging.getLogger(__name__)


class S3EventType(Enum):
    """
    REF: Supported S3 Event Types
    https://docs.aws.amazon.com/AmazonS3/latest/userguide/notification-how-to-event-types-and-destinations.html
    """
    EVENT_OBJECT_CREATED = "ObjectCreated"
    EVENT_OBJECT_REMOVED = "ObjectRemoved"
    EVENT_UNSUPPORTED = "Unsupported"


def get_s3_uri(bucket, key):
    return f"s3://{bucket}/{key}"


def get_matching_s3_objects(bucket, prefix="", suffix=""):
    """
    Generate objects in an S3 bucket.
    https://alexwlchan.net/2019/07/listing-s3-keys/

    :param bucket: Name of the S3 bucket.
    :param prefix: Only fetch objects whose key starts with
        this prefix (optional).
    :param suffix: Only fetch objects whose keys end with
        this suffix (optional).
    """
    paginator = s3_client().get_paginator("list_objects_v2")

    kwargs = {'Bucket': bucket}

    # We can pass the prefix directly to the S3 API.  If the user has passed
    # a tuple or list of prefixes, we go through them one by one.
    if isinstance(prefix, str):
        prefixes = (prefix,)
    else:
        prefixes = prefix

    for key_prefix in prefixes:
        kwargs["Prefix"] = key_prefix

        for page in paginator.paginate(**kwargs):
            try:
                contents = page["Contents"]
            except KeyError:
                break

            for obj in contents:
                key = obj["Key"]
                if key.endswith(suffix):
                    yield obj


def get_matching_s3_keys(bucket, prefix="", suffix=""):
    """
    Generate the keys in an S3 bucket.
    https://alexwlchan.net/2019/07/listing-s3-keys/

    :param bucket: Name of the S3 bucket.
    :param prefix: Only fetch keys that start with this prefix (optional).
    :param suffix: Only fetch keys that end with this suffix (optional).
    """
    for obj in get_matching_s3_objects(bucket, prefix, suffix):
        yield obj["Key"]


def bucket_exists(bucket) -> bool:
    """
    head_bucket API
    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.head_bucket

    :param bucket:
    :return bool: True if bucket exists, False otherwise
    """
    try:
        resp = s3_client().head_bucket(Bucket=bucket)
        return resp['ResponseMetadata']['HTTPStatusCode'] == 200
    except ClientError as e:
        logger.error(f"Bucket ({bucket}) not found or no permission. Exception: {e}")

    return False


def presign_s3_file(bucket: str, key: str) -> (bool, str):
    """
    Generate a presigned URL
    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.generate_presigned_url
    https://boto3.amazonaws.com/v1/documentation/api/latest/guide/s3-presigned-urls.html

    :param bucket:
    :param key:
    :return tuple (bool, str): (true, signed_url) if success, otherwise (false, error message)
    """
    try:
        return True, s3_client().generate_presigned_url('get_object', Params={'Bucket': bucket, 'Key': key})
    except ClientError as e:
        message = f"Failed to sign the specified S3 object (s3://{bucket}/{key}). Exception - {e}"
        logger.error(message)
        return False, message


def head_s3_object(bucket: str, key: str) -> (bool, dict):
    """
    head_object API
    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.head_object

    :param bucket:
    :param key:
    :return tuple (bool, dict): (true, dict object metadata) if success, otherwise (false, dict error message)
    """
    try:
        return True, s3_client().head_object(Bucket=bucket, Key=key)
    except ClientError as e:
        message = f"Failed on HEAD request the specified S3 object (s3://{bucket}/{key}). Exception - {e}"
        logger.error(message)
        return False, dict(error=message)


def restore_s3_object(bucket: str, key: str, **kwargs) -> (bool, dict):
    """
    restore_object API
    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.restore_object
    https://docs.aws.amazon.com/AmazonS3/latest/API/API_RestoreObject.html
    https://docs.aws.amazon.com/AmazonS3/latest/dev/restoring-objects.html

    :param bucket:
    :param key:
    :param kwargs:
    :return bool, dict: True if restore request is success, False otherwise with error message
    """
    allowed_tiers = ['Standard', 'Bulk']  # 'Standard' or 'Bulk' for DEEP_ARCHIVE
    tier = kwargs.get('tier', 'Bulk')
    if tier not in allowed_tiers:
        message = f"Failed restore request for the S3 object (s3://{bucket}/{key}). Allow tiers: ({allowed_tiers})"
        logger.error(message)
        return False, dict(error=message)

    days = kwargs.get('days', 7)
    requested_by = kwargs.get('email', '')
    requested_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')

    restore_request = {'Days': days, 'GlacierJobParameters': {'Tier': tier}}

    try:
        resp = s3_client().restore_object(Bucket=bucket, Key=key, RestoreRequest=restore_request)
        status_code = resp['ResponseMetadata']['HTTPStatusCode']
        logger.info(f"Requested restore for the S3 object (s3://{bucket}/{key}) with "
                    f"{days} days, {tier} tier, {requested_by} at {requested_time}. "
                    f"Response HTTPStatusCode - {status_code}")
        return True, dict(status_code=status_code)
    except ClientError as e:
        message = f"Failed restore request for the S3 object (s3://{bucket}/{key}). Exception - {e}"
        logger.error(message)
        return False, dict(error=message)


def get_s3_object(bucket: str, key: str, **kwargs) -> (bool, dict):
    """
    get_object API
    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.get_object

    :param bucket:
    :param key:
    :param kwargs:
    :return tuple (bool, dict): (true, dict object metadata) if success, otherwise (false, dict error message)
    """
    try:
        return True, s3_client().get_object(Bucket=bucket, Key=key, **kwargs)
    except ClientError as e:
        if e.response['Error']['Code'] == "304":
            return True, e.response
        message = f"Failed on GET request the specified S3 object (s3://{bucket}/{key}). Exception - {e}"
        logger.error(message)
        return False, dict(error=message)


def get_s3_object_to_bytes(bucket: str, key: str) -> bytes:
    """
    get_object_to_bytes API, with on-the-fly gzip detection and decompression

    :param bucket:
    :param key:
    :return bytes: bytes from the decompressed S3 object body
    """
    try:
        obj_body = resource('s3').Object(bucket, key).get()['Body']
        if ".gz" in key:
            obj_bytes = gzip.decompress(obj_body.read())
        else:
            obj_bytes = obj_body.read()
        return obj_bytes
    except ClientError as e:
        message = f"Failed on GET request the specified S3 object (s3://{bucket}/{key})"
        raise FileNotFoundError(message)


def get_s3_object_tagging(bucket: str, key: str):
    """
    get_object_tagging API
    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.get_object_tagging

    TODO Pass-through call

    :param bucket:
    :param key:
    :return:
    """
    return s3_client().get_object_tagging(Bucket=bucket, Key=key)


def put_s3_object_tagging(bucket: str, key: str, tagging: dict):
    """
    put_object_tagging API
    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.put_object_tagging

    TODO Pass-through call

    :param bucket:
    :param key:
    :param tagging:
    :return:
    """
    return s3_client().put_object_tagging(Bucket=bucket, Key=key, Tagging=tagging)



# -*- coding: utf-8 -*-
import logging
import sys
from importlib import util as importlib_util

logger = logging.getLogger(__name__)


def load_package_if_found(name):
    if name in sys.modules:
        logger.debug(f"{name!r} already in sys.modules")
        return True
    elif (spec := importlib_util.find_spec(name)) is not None:
        module = importlib_util.module_from_spec(spec)
        sys.modules[name] = module
        spec.loader.exec_module(module)
        logger.debug(f"{name!r} has been imported")
        return True
    else:
        logger.debug(f"can't find the {name!r} module")
        return False



# -*- coding: utf-8 -*-
"""libjson module

"""
import json
from typing import Any, Union

from libumccr.utils import load_package_if_found


def dumps(data: Any, encoder=None) -> str:
    if encoder:
        return json.dumps(data, cls=encoder)

    elif load_package_if_found("django"):
        import django.core.serializers.json as django_ser
        return json.dumps(data, cls=django_ser.DjangoJSONEncoder)

    return json.dumps(data)


def loads(data: Union[str, bytes], decoder=None) -> Any:
    if decoder:
        return json.loads(data, cls=decoder)

    return json.loads(data)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

from libumccr import __version__

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="libumccr",
    version=__version__,
    author="UMCCR and Contributors",
    author_email="services@umccr.org",
    description="UMCCR Reusable Python modules",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/umccr/libumccr",
    license="MIT",
    packages=find_packages(exclude=("tests", "docs")),
    extras_require={
        "dev": [
            "pipdeptree",
            "sphinx",
            "twine",
            "setuptools",
            "wheel",
            "pdoc3",
        ],
        "test": [
            "pytest",
            "pytest-cov",
            "flake8",
            "mockito",
        ],
        "all": [
            "Django",
        ],
    },
    install_requires=[
        "boto3",
        "botocore",
        "cachetools",
        "requests",
        "pandas",
        "gspread",
        "gspread-pandas",
        "google-auth",
    ],
    python_requires=">=3.6",
)



import logging
from unittest import TestCase, skip

from libumccr.aws import libs3

# FIXME centralise test case logger
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class LibS3UnitTests(TestCase):
    # TODO https://github.com/umccr/libumccr/issues/1
    pass


class LibS3IntegrationTests(TestCase):
    # integration test hit actual File or API endpoint, thus, manual run in most cases
    # required appropriate access mechanism setup such as active aws login session
    # uncomment @skip and hit the each test case!
    # and keep decorated @skip after tested

    @skip
    def test_get_s3_object_to_bytes(self):
        """
        python -m unittest tests.aws.test_libs3.LibS3IntegrationTests.test_get_s3_object_to_bytes
        """
        bucket = "umccr-temp-dev"
        key = "cancer_report_tables/hrd/SBJ00670__SBJ00670_MDX210005_L2100047_rerun-hrdetect.json.gz"
        result = libs3.get_s3_object_to_bytes(bucket, key)
        logger.info(f"Result type: {type(result)}, Body content: \n{result}")
        self.assertIsInstance(result, bytes)



from datetime import datetime, timezone


def get_utc_now_ts():
    return int(datetime.utcnow().replace(tzinfo=timezone.utc).timestamp())


def parse_last_modified_date(date_raw: str) -> datetime:
    """
    Date string parser for last_modified_date attribute of S3 object (event data)
    :param date_raw: date in raw string
    """
    return datetime.strptime(date_raw, '%Y-%m-%d')


def parse_lims_timestamp(date_raw: str) -> datetime:
    """
    Date string parser for timestamp column of LIMS data
    :param date_raw: date in raw string
    """
    return datetime.strptime(date_raw, '%Y-%m-%d')


def serializable_datetime(dt) -> str:
    """
    Convert to serializable datetime iso format string, e.g. use in AWS Lambda call dict return object
    :param dt:
    :return:
    """
    if isinstance(dt, str):
        return dt

    if isinstance(dt, datetime):
        _dt: datetime = dt
        return _dt.isoformat()


def folder_friendly_timestamp(dt: datetime) -> str:
    """
    Get a folder friendly timestamp of the format
    :param dt:
    :return:
    """
    return dt.strftime("%Y-%m-%d__%H-%M-%S")
